@@ -365,7 +365,7 @@ async fn count_distinct_dictionary_all_null_values() -> Result<()> {
 
     assert_snapshot!(
         batches_to_string(&results),
-        @r###"
+        @r"
     +-----+---------------+
     | cnt | count(t.num2) |
     +-----+---------------+
@@ -375,7 +375,7 @@ async fn count_distinct_dictionary_all_null_values() -> Result<()> {
     | 0   | 1             |
     | 0   | 1             |
     +-----+---------------+
-    "###
+    "
     );
 
     // Test with multiple partitions
@@ -430,13 +430,13 @@ async fn count_distinct_dictionary_mixed_values() -> Result<()> {
 
     assert_snapshot!(
         batches_to_string(&results),
-        @r###"
+        @r"
     +------------------------+
     | count(DISTINCT t.dict) |
     +------------------------+
     | 2                      |
     +------------------------+
-    "###
+    "
     );
 
     Ok(())
