@@ -19,14 +19,16 @@
 /// when the feature integration-tests is built
 #[cfg(feature = "integration-tests")]
 mod tests {
+    use std::sync::Arc;
+
     use arrow::array::Float64Array;
     use datafusion::common::record_batch;
     use datafusion::error::{DataFusionError, Result};
-    use datafusion::logical_expr::AggregateUDF;
-    use datafusion::prelude::{col, SessionContext};
-
+    use datafusion::logical_expr::{AggregateUDF, AggregateUDFImpl};
+    use datafusion::prelude::{SessionContext, col};
+    use datafusion_catalog::MemTable;
+    use datafusion_expr::{ScalarUDF, ScalarUDFImpl};
     use datafusion_ffi::tests::utils::get_module;
-    use datafusion_ffi::udaf::ForeignAggregateUDF;
 
     #[tokio::test]
     async fn test_ffi_udaf() -> Result<()> {
@@ -38,9 +40,9 @@ mod tests {
                 .ok_or(DataFusionError::NotImplemented(
                     "External table provider failed to implement create_udaf".to_string(),
                 ))?();
-        let foreign_sum_func: ForeignAggregateUDF = (&ffi_sum_func).try_into()?;
+        let foreign_sum_func: Arc<dyn AggregateUDFImpl> = (&ffi_sum_func).into();
 
-        let udaf: AggregateUDF = foreign_sum_func.into();
+        let udaf = AggregateUDF::new_from_shared_impl(foreign_sum_func);
 
         let ctx = SessionContext::default();
         let record_batch = record_batch!(
@@ -80,9 +82,9 @@ mod tests {
                 .ok_or(DataFusionError::NotImplemented(
                     "External table provider failed to implement create_udaf".to_string(),
                 ))?();
-        let foreign_stddev_func: ForeignAggregateUDF = (&ffi_stddev_func).try_into()?;
+        let foreign_stddev_func: Arc<dyn AggregateUDFImpl> = (&ffi_stddev_func).into();
 
-        let udaf: AggregateUDF = foreign_stddev_func.into();
+        let udaf = AggregateUDF::new_from_shared_impl(foreign_stddev_func);
 
         let ctx = SessionContext::default();
         let record_batch = record_batch!(
@@ -126,4 +128,69 @@ mod tests {
 
         Ok(())
     }
+
+    /// This test FFI UDFs can be used as inputs to FFI Aggregate UDFs.
+    /// Really this is a test of the Protobuf serialization and deserialization
+    /// using the TaskContextProvider. It can be demonstrated through the
+    /// UDAF accumulator arguments as an end-to-end test.
+    #[tokio::test]
+    async fn udf_as_input_to_udf() -> Result<()> {
+        let module = get_module()?;
+
+        let ffi_abs_func =
+            module
+                .create_scalar_udf()
+                .ok_or(DataFusionError::NotImplemented(
+                    "External table provider failed to implement create_scalar_udf"
+                        .to_string(),
+                ))?();
+        let foreign_abs_func: Arc<dyn ScalarUDFImpl> = (&ffi_abs_func).into();
+        let abs_udf = ScalarUDF::new_from_shared_impl(foreign_abs_func);
+
+        let ctx = SessionContext::new();
+        ctx.deregister_udf("abs");
+
+        let ffi_sum_func =
+            module
+                .create_sum_udaf()
+                .ok_or(DataFusionError::NotImplemented(
+                    "External table provider failed to implement create_udaf".to_string(),
+                ))?();
+        let foreign_sum_func: Arc<dyn AggregateUDFImpl> = (&ffi_sum_func).into();
+
+        let udaf = AggregateUDF::new_from_shared_impl(foreign_sum_func);
+
+        // We need at least 2 record batches so we get an accumulator
+        let ctx = SessionContext::default();
+        let rb1 = record_batch!(
+            ("a", Int32, vec![1, 2, 2, 4, 4, 4, 4]),
+            ("b", Float64, vec![-1.0, 2.0, -2.0, 4.0, -4.0, -4.0, -4.0])
+        )
+        .unwrap();
+        let rb2 = rb1.clone();
+
+        let table = Arc::new(MemTable::try_new(rb1.schema(), vec![vec![rb1, rb2]])?);
+
+        let df = ctx.read_table(table)?;
+
+        let df = df
+            .aggregate(
+                vec![col("a")],
+                vec![udaf.call(vec![abs_udf.call(vec![col("b")])]).alias("sum_b")],
+            )?
+            .sort_by(vec![col("a")])?;
+
+        df.clone().show().await?;
+
+        let result = df.collect().await?;
+
+        let expected = record_batch!(
+            ("a", Int32, vec![1, 2, 4]),
+            ("sum_b", Float64, vec![2.0, 8.0, 32.0])
+        )?;
+
+        assert_eq!(result[0], expected);
+
+        Ok(())
+    }
 }
